%% Solutions for Practical Exercise 2 of the ICN Matlab Course 2016
%  Daniel Bush, UCL (2016) d.bush@ucl.ac.uk

%  Part One: Write a function that returns information about different
%  countries. The function should take the name of a country as a single
%  string input, and return a single structure named Info that contains
%  the following fields as an output: Population, with a value equal to
%  the population of that country; Capital, with a string that contains
%  the name of the capital of that country; Language, with a string that
%  contains the name of the official language of that country; and Area,
%  with a value equal to the area of that country in square kilometres.
%  Choose any five or more countries that this function will return data
%  for. If the input string does not match the name of any of your chosen
%  countries, then ask the function to print the following text on screen:
%  Sorry, I do not have any information about that country

%  This problem can be solved using a simple switch / case statement on the
%  variable containing the name of the country, as shown below. Copy and
%  paste the code below into a new script, save it as 'CountryData' in your
%  current folder, and then try typing the following into the command
%  window (with no semi colon so that the output is displayed on screen):
%  Info = CountryData('Spain')

function[Info]  = CountryData(Name)
% Function to return information about the country provided as input Name,
% including the population (in millions), capital city, primary language
% and area is square kilometeres
% Daniel Bush, UCL (2016) d.bush@ucl.ac.uk

% First, assign some memory
Info = struct;          % A function must always generate the output 
                        % specified in square brackets after the 'function'
                        % call above, or Matlab will throw an error. Hence,
                        % it's best to create empty variables at the start 
                        % of the script corresponding to each of the
                        % outputs, and then just add values to those
                        % variables later in the script. Then, if anything
                        % goes wrong, the function will still work, it will
                        % just return empty variables

switch Name
    case {'Sweden','Sverige'}       % You can list multiple inputs that 
                                    % will run the same line of code inside
                                    % curly brackets
        Info.Population     = 9.92;
        Info.Capital        = 'Stockholm';
        Info.Language       = 'Swedish';
        Info.Area           = 450295;
        
    case {'Lithuania',''}
        Info.Population     = 2.86;
        Info.Capital        = 'Vilnius';
        Info.Language       = 'Lithuanian';
        Info.Area           = 65300;
        
    case {'Spain','Espana'}
        Info.Population     = 46.4;
        Info.Capital        = 'Madrid';
        Info.Language       = 'Spanish';
        Info.Area           = 505990;
        
    case {'Hungary',''}
        Info.Population     = 9.86;
        Info.Capital        = 'Budapest';
        Info.Language       = 'Hungarian';
        Info.Area           = 93030;
        
    case {'Czech Republic',''}
        Info.Population     = 10.6;
        Info.Capital        = 'Prague';
        Info.Language       = 'Czech';
        Info.Area           = 78866;
        
    otherwise        
        disp('Sorry, I do not have any information about that country')
end
clear Name          % Clear any variables that we no longer need



%% Part Two: Write a function that converts a distance in metres to one of 
% several other units. The function should take two inputs  one numerical
% value that corresponds to a length, in metres; and one string that
% corresponds to another unit of length - either inches, in; feet, ft;
% millimetres, mm; or centimetres, cm. The function should return one
% numerical output, corresponding to the input distance converted into the
% chosen units, and print text describing the conversion on screen. For
% example, if I were to enter a distance of 1 as an input and a unit of
% feet, the function would return an output variable with a value of
% 3.281 and print the following string on the screen: 1 metre is equal to
% 3.281 feet

% Again, this can be solved with a simple switch / case statement on the
% unit input. In this case, however, the different options specify the
% operation that is performed on the distance input, as well as a
% customised output string that is generated using disp and num2str

function[converted] = DistanceConverter(distance,units)
% Function to convert distance measured in metres specified by the 
% 'distance' input to one of several other units specified by the 'units' 
% input
% Daniel Bush, UCL (2016) d.bush@ucl.ac.uk

switch units
    case {'inches','in'}
        converted = distance * 39.3701;
        disp([num2str(distance,4) ' metre(s) is equal to ' num2str(converted,4) ' inches']);
    case {'feet','ft'}
        converted = distance * 3.281;
        disp([num2str(distance,4) ' metre(s) is equal to ' num2str(converted,4) ' feet']);
    case {'millimetres','mm'}
        converted = distance * 1000;
        disp([num2str(distance,4) ' metre(s) is equal to ' num2str(converted,4) ' millimetres']);
    case {'centimetres','cm'}
        converted = distance * 100;
        disp([num2str(distance,4) ' metre(s) is equal to ' num2str(converted,4) ' centimetres']);
    otherwise
        disp('Units not recognised, returning distance in metres')
        converted = distance;
end



%% Part Three: Write a script that first generates a 100 row x 101 column 
% matrix named Data containing random numbers that are normally
% distributed with a mean of 700 and a standard deviation of 100

% We'll start here. The function 'randn' generates normally distributed
% random numbers with a mean of zero and a standard deviation of one. We
% can adjust the mean by adding a constant to the output, and adjust the
% standard deviation by multiplying the output by a constant. So we have:

Data    = 700 + randn(100,101) .* 100;

% You can check this by computing the mean and standard deviation of the
% values in Data using 'mean' and 'std'. Remember that, without any
% additional inputs, these functions will give you the mean and standard
% deviation of each column separately. To get the mean and standard
% deviation for all the values in Data, use Data(:) to collapse the matrix
% into a single column, i.e. try typing:

mean(Data(:))
std(Data(:))

% Then complete the following steps: 
% 1. Replace any values that are less than 500 with a value of 500 

% There are several ways to do this, using the functions and techniques you
% have learned so far. The most obvious option is to use 'find' to find all
% of the entries in Data that are less than 500, and then replace those
% entries with the value 500, like so:

indices = find(Data<500);
Data(indices) = 500;

% Note that we find the indices in this case, instead of finding the row 
% and column, which could be done with:

[row col] = find(Data<500);

% This is because you cannot replace a set of values that you have the row 
% column index for in one step. Try it and see what happens by looking at 
% Data in the variable editor:

Data(row,col) = 500;

% There is a better way to do this, however, using logical indexing. Matlab
% is even suggesting that you use logical indexing, by highlighting the
% 'indices' in Data(indices) = 500; above - hover your mouse over the
% highlighted word and see what it says. We can find and change all values
% that obey some logical expression in one step using logical indexing,
% like so:

Data(Data<500) = 500;

% In this case the 'Data<500' in brackets finds all values of Data that are
% less than 500, and then the rest of the statement tells Matlab to replace
% all of those entries in Data with the number 500. 

% In either case, you can check it has worked by asking for the minimum
% value in Data before and after the replacement, using:

min(Data(:))

% 2. Add 100 to the 20th, 25th, 30th and 35th rows 

% This should be straightforward by now, simply use the indices to access
% those specific rows of Data and then add 100 to their current value:

Data([20 25 30 35],:)   = Data([20 25 30 35],:) + 100;

% 3. Multiply the 90 to 100th rows by 1.5 

% As above, this should be fairly simple. Remember that you can use the
% colon operator to list all of the integer values between two numbers, in
% which case you don't need the square brackets:

Data(:,90:100)  = Data(:,90:100) .* 1.5;

% 4. Change the first column to the numbers 1 to 100 

Data(:,1)       = 1 : 100;

% 5. Compute the mean of each row, excluding the first column, and store it
% as a variable named MeanRT 

% We can compute the mean of each row, rather than column (which is the
% default), by giving the 'mean' function a second input to tell it to
% compute the mean across a different dimension of the matrix (see the mean
% help information for more examples). Then we just need to only ask for
% the mean of the second to final columns of Data:

MeanRT          = mean( Data(:,2:end), 2);

% 6. Generate a 100 row x 1 column variable named Conditions that contains
% fifty values of 1 and fifty values of 2 in a randomised order 

% Again, there are several ways to do this, but the most straightforward is
% to use randperm, which is also a good function to get to grips with as it
% is very useful for randomising the order of conditions or trials in
% experimental scenarios. First, we generate a variable that has fifty ones
% and fifty twos in it:

Conditions      = [ones(50,1) ; 2*ones(50,1)];

% Them we use 'randperm' to randomise the order of those entries, by moving
% each entry of conditions to another row at random. The random row numbers
% are generated by randperm(100) or, more neatly,
% ranperm(length(Conditions)), which automatically detects the number of
% rows in Conditions. We can do this all in one line, like so:

Conditions      = Conditions(randperm(length(Conditions)));

% 7. Imagine that the MeanRT variable contains mean reaction time data for
% 100 participants in a behavioural task who were divided into two 
% conditions, corresponding to the two different values in Conditions. 
% Compute the standard error of the MeanRT values for each condition 
% separately and store them as variables named StdErr1 and StdErr2, 
% respectively 

% Hopefully, by now, this should be straightforward, if you can remember
% the equation for standard error! It is the standard deviation divided by
% the square root of the sample size. So, in this case, we can first use
% logical indexing to find all of the MeanRT values in each 'condition',
% like so (try copying the following into the command window and running 
% them):

MeanRT(Conditions==1)
MeanRT(Conditions==2)

% We can also add up all the values of Conditions==1 and Conditions==2 to
% get the sample size for each (as these expressions are equal to 1 where
% true and 0 where false, so adding up the whole array gives you the number
% of cases where the expression is true):

sum(Conditions==1)
sum(Conditions==2)

% Then we can combine these expressions with the std and sqrt functions to 
% get the standard errors:

StdErr1     = std(MeanRT(Conditions==1)) / sqrt(sum(Conditions==1));
StdErr2     = std(MeanRT(Conditions==2)) / sqrt(sum(Conditions==2));

% 8. Perform a two sample t-test comparing the values in MeanRT between
% participants in each condition. Store the t-statistic as a variable named
% t_stat and the corresponding p value as a variable named p_value

% You will probably have had to Google 'two sample t test matlab' to work
% out how to complete this part. The function you need is called ttest2,
% and if you look at the help information for that function, you can see
% that you just provide the two arrays of data as two inputs to that
% function. The function can provide multiple outputs, however, and to get
% hold of the t-statistic, you need to get all four outputs, as the
% t-statistic is stored as a field of the final 'stats' output. To get an 
% idea of what I mean, have a look at the ttest2 help information, and 
% search for 'tstat'

% We already saw above how to get the raw MeanRT values for each condition
% using logical indexing, so now we can just run:

[h p ci stats] = ttest2(MeanRT(Conditions==1),MeanRT(Conditions==2))

% Which provides the p value as the variable 'p' and the t-statistic as the
% 'tstat' field in the 'stats' output. Job done!